/*---------------------------------------------------------------------\
|                                                                      |
|                     _     _   _   _     __     _                     |
|                    | |   | | | \_/ |   /  \   | |                    |
|                    | |   | | | |_| |  / /\ \  | |                    |
|                    | |__ | | | | | | / ____ \ | |__                  |
|                    |____||_| |_| |_|/ /    \ \|____|                 |
|                                                                      |
|                             core library                             |
|                                                                      |
|                                         (C) SUSE Linux Products GmbH |
\----------------------------------------------------------------------/

  File:       config.h

  Author:     Marius Tomaschewski
  Maintainer: Marius Tomaschewski

/-*/
/**
 * @file   config.h
 * @brief  LiMaL configuration file.
 *
 * This is config file provides compilation time definitions
 * like the version of the library, and the declaration of
 * the main LiMaL namespace.
 */
/**
 * @mainpage
 * The LiMaL core library provides common utilities, that can be
 * used to implement specific LiMaL libraries called "pluglibs".
 *
 * Further, it provides Logging utilities, several common Exception
 * types and a callback interface, inclusive bindings for applications 
 * implemented in scripting languages (currently perl) using SWIG.
 *
 */
#ifndef LIMAL_CONFIG_H
#define LIMAL_CONFIG_H 1

#include  <blocxx/BLOCXX_config.h>


/**
 * @brief LiMaL library version.
 */
#ifndef LIMAL_LIB_VERSION
#define LIMAL_LIB_VERSION			"1.5.0"
#endif


/**
 * @brief LiMaL API/namespace version.
 */
#ifndef LIMAL_API_VERSION
#define LIMAL_API_VERSION			1
#endif

 
/**
 * @brief Helper macros to define a namespace with version.
 */
#ifndef LIMAL_DEFINE_NAMESPACE_CAT 
#define LIMAL_DEFINE_NAMESPACE_CAT(name,ver)   name ## ver 
#endif
#ifndef LIMAL_DEFINE_NAMESPACE 
#define LIMAL_DEFINE_NAMESPACE(name,ver)       LIMAL_DEFINE_NAMESPACE_CAT(name,ver) 
#endif


/*
 * Definition of the main LiMaL namespace
 * with appended API version number.
 */
#ifndef LIMAL_NAMESPACE 
#define LIMAL_NAMESPACE	\
	LIMAL_DEFINE_NAMESPACE(limal, LIMAL_API_VERSION)
#endif /* LIMAL_NAMESPACE */


/**
 * @brief The main LiMaL namespace.
 *
 * The @c limal namespace is only a alias to the real namespace
 * @c LIMAL_NAMESPACE, that is defined in the config.h file
 * with appended API version number.
 *
 * The full namespace name should be used if something is declared
 * in the namespace. For example:
 * @code
 * namespace LIMAL_NAMESPACE
 * {
 *     class MyLiMaLClass
 *     {
 *        MyLiMaLClass();
 *        // ...
 *     };
 *
 *     MyLiMaLClass::MyLiMaLClass()
 *     {
 *     }
 * }
 * @endcode
 *
 * The @c limal alias is to qualify e.g. a class name.
 * @code
 * namespace
 * {
 *    void somefunction(const limal::Logger &logger);
 * }
 * @endcode
 */
#ifdef    __cplusplus
namespace LIMAL_NAMESPACE
{
}
#ifndef DOXYGEN_SKIP
namespace limal = LIMAL_NAMESPACE;
#endif
#endif /* __cplusplus */


#ifdef    __cplusplus
extern "C" {
#endif /* __cplusplus */


/**
 * @brief LiMaL library version string function.
 * @returns library version string
 */
const char * limal_lib_version();


/**
 * @brief LiMaL API version number function.
 * @returns API version number
 */
unsigned int limal_api_version();


#ifdef    __cplusplus
}
#endif /* __cplusplus */

#endif /* LIMAL_CONFIG_H */
/* vim: set ts=8 sts=8 sw=8 ai noet: */
